package meerkat.voting.controller.callbacks;

import meerkat.protobuf.Voting.UIElement;
import meerkat.voting.controller.commands.*;
import meerkat.voting.controller.commands.ControllerCommand;
import meerkat.voting.ui.VotingBoothUI.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.LinkedBlockingQueue;

public class CastOrAuditCallback extends ControllerCallback<FinalizeBallotChoice> {
    protected final static Logger logger = LoggerFactory.getLogger(CastOrAuditCallback.class);
    protected final UIElement unrecognizedFinalizeResponseMessage;

    public CastOrAuditCallback(int requestId,
                               long ballotSerialNumber,
                               LinkedBlockingQueue<ControllerCommand> controllerQueue,
                               UIElement unrecognizedFinalizeResponseMessage) {
        super(requestId, ballotSerialNumber, controllerQueue);
        this.unrecognizedFinalizeResponseMessage = unrecognizedFinalizeResponseMessage;
    }

    @Override
    public void onSuccess(FinalizeBallotChoice result) {
        if (result == FinalizeBallotChoice.CAST) {
            enqueueCommand(new CastCommand(getRequestIdentifier(), getBallotSerialNumber()));
        }
        else if (result == FinalizeBallotChoice.AUDIT) {
            enqueueCommand(new AuditCommand(getRequestIdentifier(), getBallotSerialNumber()));
        }
        else {
            logger.error("CastOrAuditCallback got an unrecognized response: " + result);
            enqueueCommand(new ReportErrorCommand(getRequestIdentifier(),
                    getBallotSerialNumber(),
                    unrecognizedFinalizeResponseMessage));
        }

    }

    @Override
    public void onFailure(Throwable t) {
        logger.error("CastOrAuditCallback got a failure: " + t);
        enqueueCommand(new ReportErrorCommand(getRequestIdentifier(),
                getBallotSerialNumber(),
                unrecognizedFinalizeResponseMessage));
    }
}



