package meerkat.voting.controller.callbacks;

import meerkat.protobuf.Voting.*;
import meerkat.voting.controller.commands.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.concurrent.LinkedBlockingQueue;

/**
 * Created by hai on 11/04/16.
 */
public class ChannelChoiceCallback extends ControllerCallback<List<BallotAnswer>> {
    protected final static Logger logger = LoggerFactory.getLogger(ChannelChoiceCallback.class);
    protected final UIElement unsuccessfulChannelChoiceMessage;

    public ChannelChoiceCallback(int requestId,
                                 long ballotSerialNumber,
                                 LinkedBlockingQueue<ControllerCommand> controllerQueue,
                                 UIElement unsuccessfulChannelChoiceMessage) {
        super(requestId, ballotSerialNumber, controllerQueue);
        this.unsuccessfulChannelChoiceMessage = unsuccessfulChannelChoiceMessage;
    }

    @Override
    public void onSuccess(List<BallotAnswer> result) {
        logger.debug("callback for channel choice returned success");
        enqueueCommand(new ChannelDeterminedCommand(getRequestIdentifier(), getBallotSerialNumber(), result));
    }

    @Override
    public void onFailure(Throwable t) {
        if (t instanceof VoterCancelException) {
            logger.debug("ChannelChoiceCallback got a cancellation response");
            enqueueCommand(new RestartVotingCommand(getRequestIdentifier(), getBallotSerialNumber()));
        }
        else {
            logger.error("channel choice initiated a failure: " + t);
            enqueueCommand(new ReportErrorCommand(getRequestIdentifier(),
                    getBallotSerialNumber(),
                    unsuccessfulChannelChoiceMessage));
        }
    }
}
