package meerkat.voting.controller.callbacks;

import jdk.nashorn.internal.runtime.regexp.joni.exception.ValueException;
import meerkat.voting.controller.commands.ControllerCommand;
import meerkat.voting.controller.commands.RestartVotingCommand;
import meerkat.voting.controller.commands.RetryEncryptAndCommitBallotCommand;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.LinkedBlockingQueue;

public class EncryptionFailedCallback extends ControllerCallback<Integer> {
    protected final static Logger logger = LoggerFactory.getLogger(EncryptionFailedCallback.class);

    public EncryptionFailedCallback(int requestId,
                                    long ballotSerialNumber,
                                    LinkedBlockingQueue<ControllerCommand> controllerQueue) {
        super(requestId, ballotSerialNumber, controllerQueue);
    }

    @Override
    public void onSuccess(Integer result) {
        logger.debug("callback for voting returned success");
        int res = result.intValue();
        if (res == 0) {
            logger.debug("voter chose to retry encryption");
            enqueueCommand(new RetryEncryptAndCommitBallotCommand(getRequestIdentifier(), getBallotSerialNumber()));
        }
        else if (res == 1) {
            logger.debug("voter chose to cancel the vote");
            enqueueCommand(new RestartVotingCommand(getRequestIdentifier(), getBallotSerialNumber()));
        }
        else {
            onFailure(new ValueException("EncryptionFailedCallback got an unknown result (" + res + ")"));
        }
    }

    @Override
    public void onFailure(Throwable t) {
        logger.error("Error message execution initiated a failure: " + t);
        enqueueCommand(new RestartVotingCommand(getRequestIdentifier(), getBallotSerialNumber()));
    }
}
